package treeparser;

import roma.ConsoleReader;
import roma.Node;
import java.io.StringReader;
import java.io.IOException;

/*============================================================================
			 DEPTH FIRST VISIT OF A TREE (iterative version)
Grammar for the input tree:  char ::= '0' | '1' | ... | '9'
                             tree ::= char | - char tree | (tree char tree)
Internal tree:               tree = Num  |  Num x tree  |  tree x Num x tree
                                   (Leaf)     (Unary)          (Binary)
Grammar for the output tree: tree ::= n | (n.tree) | (tree.n.tree)
=========================================================================== */

public class DepthFirstVisit{
    /**
     * Trasforma una stringa rappresentante un albero secondo la grammatica
     * fornita in un albero di oggetti di tipo Node
     * @param string: stringa di cui effettuare il parsing
     * @return la radice dell'albero prodotto dal processo di parsing
     * @throws IOException
     */
   public static Node parseTree(String string)
      throws IOException, ParseTreeException{
      StringReader sr = new StringReader(string);
      return parseTree(sr);	
   }
	
   public static Node parseTree(StringReader string)
      throws IOException, ParseTreeException{
      char c = (char)string.read();
      if ( ('0' <= c ) && ( c <= '9') ) {
         return new LeafNode(new Integer(c - 48));
      }else
      if ( c == '-'){
         char c2        = (char)string.read();
         Integer value  = new Integer(c2 - 48);
         Node cntTree   = parseTree(string);
         UnaryNode node = new UnaryNode(value, cntTree);
         return node;
      }else
      if ( c == '(' ){
         Node leftTree   = parseTree(string);
         char c2         = (char)string.read();
         Integer value   = new Integer(c2 - 48);
         Node rightTree  = parseTree(string);
         BinaryNode node = new BinaryNode(value, leftTree, rightTree);
         c2              = (char)string.read();
         return node;
      } else{
      throw new ParseTreeException();
      }
   }
	
   /**
   * Visita in profondit dell'albero
   * @param tree albero da visitare in profondit
   * @return la "traduzione" dell'albero nella grammatica specificata
   */
   public static String depthFirstVisit(Node tree){
      TreeStack treestack = new TreeStack();
      StringBuffer buffer = new StringBuffer();
      treestack.push(tree);
      while(!treestack.isEmpty()){
         Node topNode = treestack.pop();
         if ( topNode instanceof LeafNode ){
            buffer.append(topNode.value + " ");
         }else
         if ( topNode instanceof UnaryNode ){
            LeafNode ln = new LeafNode(topNode.value);
            treestack.push(ln);
            treestack.push(((UnaryNode)topNode).getChild());
         }else
         if ( topNode instanceof BinaryNode ){
            LeafNode ln = new LeafNode(topNode.value);
            treestack.push(ln);
            treestack.push(topNode.right);
            treestack.push(topNode.left);
         }			
      }
      return buffer.toString();
   }
    
   /**
     * @param args the command line arguments
   */
    public static void main(String[] args) {
       ConsoleReader console = new ConsoleReader(System.in);
       System.out.println("Input a tree according to the given grammar above,"
                         +"please.\n"); 	
       String inputLine = console.readLine();
       Node tree = null;
       try {
           tree = parseTree(inputLine);
       } catch ( IOException e) {
            e.printStackTrace();
       } catch ( ParseTreeException pex ){
            System.out.println(pex.getMessage());
       }
       if (tree != null){
            System.out.println("\nTree in input:\n" + tree);
            System.out.println("\nDepth first visit of the tree:");
            System.out.println(depthFirstVisit(tree));
       }
   }
}
/* input:    output:
------------------------------------------------------------------------------
             Input a tree according to the given grammar above, please.
             (-231(-564-78))

             Tree in input:
             ((2.3).1.((5.6).4.(7.8)))

             Depth first visit of the tree:
             3 2 6 5 8 7 4 1
--------------------------------------------------------------------------- */

